/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2006 Torus Knot Software Ltd
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        CHCDemo.cpp
    \brief
        Shows OGRE's CHCDemo feature, and using it to drive a reflection.
*/

#include "ExampleApplication.h"
#include "ExampleFrameListener.h"

#include "betagui.h"

//#define _STANDARD



class CHCDemoFrameListener : public ExampleFrameListener
{
protected :

	BetaGUI::GUI *mGui;
	BetaGUI::Window* mWin;

	Ogre::Camera* mCamera1;
	Ogre::Camera* mCamera2;

public:
    CHCDemoFrameListener(RenderWindow* window, Camera* maincam)
        :ExampleFrameListener(window, maincam) 
    {

		// augment move speed
		mMoveSpeed = mMoveSpeed * 10;;

		/*
		mGui = new BetaGUI::GUI(window);


		BetaGUI::Style* mBetaGUIStyle = mGui->createStyle("myBetagui");

		mBetaGUIStyle->mWindow_Background = "bgui.window";
		mBetaGUIStyle->mFontFace = "MgOpen";
		mBetaGUIStyle->addStyle("button", "background: bgui.button; height: 20; text-align: center; font-size: 14; decal-size: 10;");
		mBetaGUIStyle->addStyle("titlebar", "background: bgui.titlebar; height: 20; font-size: 15;");
		mBetaGUIStyle->addStyle("textinput", "background: bgui.textinput; height: 20; font-size: 12; decal-size: 5;");

		mGui->createMousePointer("bgui.pointer", 32, 32);

		mWin = mGui->createWindow(0.5, 0.5, 200, 100, "myBetagui", BetaGUI::WFT_RESIZE_AND_MOVE, "Nice", 0);
		*/



		String cullmode(
			//"VIEW_FRUSTUM"			// Octree view frustum culling with 3 type (Full, NONE, PARTIAL)
			//"VIEW_FRUSTUM_MASK"			// Octree view frustum culling with 3 type (Full, NONE, PARTIAL)
			//"VIEW_FRUSTUM_MASK_TEMPORAL" // Octree view frustum culling with 3 type (Full, NONE, PARTIAL)
			//"VIEW_FRUSTUM_DIRECT"		// Octree view frustum culling with  2 type(Visible, Invisible)
			//"VIEW_FRUSTUM_RADAR"		// Octree view frustum culling with  2 type(Visible, Invisible)
			//"VIEW_FRUSTUM_COMPLEX"	// Octree view frustum culling with 3 type (Full, NONE, PARTIAL)
			//"STOP_AND_WAIT"			// Query all visible before drawing it, meaning it's waiting for query results (slow, debug purpose)
			"CHC"						// render whole tree that was visible at previous frame and test other part of tree and leafs.
			//"CHC_CONSERVATIVE"		// Try to use previous frame info to reduce query time (WIP)
			);


		SceneManager *sm = mCamera->getSceneManager();

		assert(sm);
		sm->setOption (String("setCullingMode"), &cullmode);


		bool show;

		//show = true;
		show = false;
		sm->setOption (String("ShowOctree"), &show);

		//show = true;
		show = false;
		sm->setOption (String("CullDebug"), &show);

		show = true;
		//show = false;
		sm->setOption (String("CullDebugText"), &show);

		//show = true;
		//show = false;
		//mCamera->getSceneManager()->setOption (String("CullCamera"), &show);

		//default is   (-10000, -10000, -10000, 
		//				 10000,  10000,  10000) 
		AxisAlignedBox aabox(-20000, -20000, -20000, 
			20000,  20000,  20000);
		sm->setOption (String("Size"), &aabox);

		int valInt;
		//default is  8; 
		valInt = 64;
		sm->setOption (String("Depth"), &valInt);

		OverlayElement* guiDbg = OverlayManager::getSingleton().getOverlayElement("Core/DebugText");
		guiDbg->show();

		//mCamera->getSceneManager()->showBoundingBoxes(true);		

		mCamera1 = mCamera;
		mCamera2 = sm->createCamera("SecondTestVis");
		mCamera2->setPosition(mCamera->getPosition() - Vector3(0, 200, 0));

		// Create an entity from a model (will be loaded automatically)
		mKnotEnt = sm->createEntity("StaticOccluder_Knot", "knot.mesh");
		mKnotEnt->setMaterialName("Examples/TextureEffect2");

		mHouseEnt = sm->createEntity("StaticOccluder_House", "tudorhouse.mesh");

		mCurrNumClones = 0;
		addStaticEntities();
		addStaticHouse();
    }
    bool frameStarted(const FrameEvent& evt)
    {
        if( ExampleFrameListener::frameStarted(evt) == false )
			return false;

		if (mTimeUntilNextToggle <= 0)
		{
			String cullmode;


#define MY_HIT_KEY_ONCE(A,B) if(mKeyboard->isKeyDown(A))		\
			{												\
			cullmode = B;									\
			}

					MY_HIT_KEY_ONCE(OIS::KC_1, "VIEW_FRUSTUM_DIRECT");
					MY_HIT_KEY_ONCE(OIS::KC_2, "VIEW_FRUSTUM");
					MY_HIT_KEY_ONCE(OIS::KC_3, "VIEW_FRUSTUM_MASK");
					MY_HIT_KEY_ONCE(OIS::KC_4, "VIEW_FRUSTUM_MASK_TEMPORAL");
					MY_HIT_KEY_ONCE(OIS::KC_5, "STOP_AND_WAIT");
					MY_HIT_KEY_ONCE(OIS::KC_6, "CHC");
					MY_HIT_KEY_ONCE(OIS::KC_7, "CHC_CONSERVATIVE");


#undef MY_HIT_KEY_ONCE

			if(!cullmode.empty())
			{
				mCamera->getSceneManager()->setOption (String("CurrentOptionCamera"), mWindow->getViewport(0)->getCamera());
				mCamera->getSceneManager()->setOption (String("setCullingMode"), &cullmode);
				mTimeUntilNextToggle = 1.0f;
			}

			if(mKeyboard->isKeyDown(OIS::KC_8))
			{
				//show = true;
				bool show;
				show = mCamera->getSceneManager()->getShowBoundingBoxes();
				show = !show;
				mCamera->getSceneManager()->showBoundingBoxes(show);
				mTimeUntilNextToggle = 1;
			}

			//;		

			if(mKeyboard->isKeyDown(OIS::KC_9))
			{
				//show = true;
				bool show;
				mCamera->getSceneManager()->getOption (String("ShowOctree"), &show);;
				show = !show;
				mCamera->getSceneManager()->setOption (String("ShowOctree"), &show);
				mTimeUntilNextToggle = 1;
			}

			if(mKeyboard->isKeyDown(OIS::KC_0))
			{
				//show = true;
				bool show;
				mCamera->getSceneManager()->getOption (String("CullDebug"), &show);;
				show = !show;
				mCamera->getSceneManager()->setOption (String("CullDebug"), &show);
				mTimeUntilNextToggle = 1;
			}


			if(mKeyboard->isKeyDown(OIS::KC_C))
			{
				Viewport *v = mWindow->getViewport(0);

				if (v->getCamera() == mCamera1)
				{
					v->setCamera(mCamera2);
					mCamera = mCamera2;
				}
				else
				{
					v->setCamera(mCamera1);
					mCamera = mCamera1;
				}
				mTimeUntilNextToggle = 1;
			}

			if(mKeyboard->isKeyDown(OIS::KC_ADD))
			{
				addStaticEntities();
				mTimeUntilNextToggle = 1;
			}

			if(mKeyboard->isKeyDown(OIS::KC_SUBTRACT))
			{
				removeStaticEntities();
				mTimeUntilNextToggle = 1;
			}

			if(mKeyboard->isKeyDown(OIS::KC_K))
			{
				switchToShaders();
				mTimeUntilNextToggle = 1;
			}

		}

		{
			static std::set<Entity*> lastEnts;

			if (!lastEnts.empty())
			{
				// Reset last set
				for (std::set<Entity*>::iterator lasti = lastEnts.begin();
					lasti != lastEnts.end(); ++lasti)
				{
					(*lasti)->setMaterialName("Examples/TextureEffect2");
				}
				lastEnts.clear();
			}

			if(mKeyboard->isKeyDown(OIS::KC_I))
			{
				static IntersectionSceneQuery* intersectionQuery = mCamera->getSceneManager()->createIntersectionQuery();
				


				IntersectionSceneQueryResult& results = intersectionQuery->execute();
				for (SceneQueryMovableIntersectionList::iterator mov = results.movables2movables.begin();
					mov != results.movables2movables.end(); ++mov)
				{
					SceneQueryMovableObjectPair& thepair = *mov;
					if (thepair.first->getMovableType() == "Entity")
					{
						Entity* ent = static_cast<Entity*>(thepair.first);
						lastEnts.insert(ent);
						ent->setMaterialName("Examples/OgreLogo");

					}
					if (thepair.second->getMovableType() == "Entity")
					{
						Entity* ent = static_cast<Entity*>(thepair.second);
						lastEnts.insert(ent);
						ent->setMaterialName("Examples/OgreLogo");

					}
				}
			}

			if(mKeyboard->isKeyDown(OIS::KC_O))
			{
				static RaySceneQuery* rayQuery  = mCamera->getSceneManager()->createRayQuery(mCamera->getCameraToViewportRay(0.5, 0.5));
				//rayQuery->setSortByDistance(true, 1);

				rayQuery->setRay(mCamera->getCameraToViewportRay(0.5, 0.5));

				RaySceneQueryResult& results = rayQuery->execute();
				for (RaySceneQueryResult::iterator mov = results.begin();
					mov != results.end(); ++mov)
				{
					if (mov->movable)
					{
						if (mov->movable->getMovableType() == "Entity")
						{
							Entity* ent = static_cast<Entity*>(mov->movable);
							lastEnts.insert(ent);
							ent->setMaterialName("Examples/OgreLogo");

						}
					}
				}
			}

			const OIS::MouseState &ms = mMouse->getMouseState();


			//mGui->injectMouse( ms.X.abs,  ms.Y.abs, ms.buttonDown( OIS::MB_Left ));
			String scnDebugText;
			mCamera->getSceneManager()->getOption (String("DebugText"), &scnDebugText);

			mDebugText = scnDebugText;

		}
        return true;
	}


#ifdef _DEBUG
	static const unsigned int mNumClone = 5;
	static const unsigned int mHouseDist = 2050;
#else  //_DEBUG
	static const unsigned int mNumClone = 2000;
	static const unsigned int mHouseDist = 1050;
#endif //_DEBUG
	unsigned int mCurrNumClones;

	// Create an entity from a model (will be loaded automatically)
	Entity* mKnotEnt;
	// Create an entity from a model (will be loaded automatically)
	Entity* mHouseEnt;




	void addStaticHouse()
	{

		StaticGeometry* s = mCamera->getSceneManager()->createStaticGeometry("StaticOccluder");

		s->setRegionDimensions(Vector3(1950.0,1950.0,1950.0));
		// Set the region origin so the centre is at 0 world
		s->setOrigin(Vector3(0, 0, 0));

		int n = 0;
		for (int x = -19500; x < 19500; x += mHouseDist)
		{
			for (int z = -19500; z < 19500; z += mHouseDist)
			{
				Vector3 pos(
					x + Math::RangeRandom(-25, 25), 
					80, 
					z + Math::RangeRandom(-25, 25));
				Quaternion orientation;
				orientation.FromAngleAxis(
					Degree(Math::RangeRandom(0, 359)),
					Vector3::UNIT_Y);
				Vector3 scale(
					1, Math::RangeRandom(0.85, 1.15), 1);

				// Clone House
				char cloneName[255];
				// StaticOccluder in the name is the way for the Scene manager to know how to handle it.
				sprintf(cloneName, "StaticOccluder_House%d", n++);
				s->addEntity(mHouseEnt->clone(cloneName), pos, orientation, scale);
			}

		}

		s->build();
	}

	void addStaticEntities()
	{
		if (mCurrNumClones < 100000)
		{
			// Attach the entity to the root of the scene
			SceneNode* rootNode = mCamera->getSceneManager()->getRootSceneNode();

			// Add a whole bunch of extra entities
			Entity *cloneEnt;
			for (unsigned int n = 0; n < mNumClone; ++n)
			{
				// Clone knot
				char cloneName[255];
				// StaticOccluder in the name is the way for the Scene manager to know how to handle it.
				sprintf(cloneName, "StaticOccluder_Knot%d", n + mCurrNumClones);

				// Create a new node under the root
				SceneNode* node = mCamera->getSceneManager()->createSceneNode(cloneName);

				// Random Scale
				Vector3 nodeScale;
				const Real scaleFactor = Math::UnitRandom() * 2.0;
				nodeScale.x = scaleFactor;
				nodeScale.y = scaleFactor;
				nodeScale.z = scaleFactor;
				node->setScale(nodeScale);

				// Random translate
				Vector3 nodePos;
				const Real posFactor = 19500.0;
				nodePos.x = Math::SymmetricRandom() * posFactor;
				nodePos.y = Math::SymmetricRandom() * posFactor / 10;
				nodePos.z = Math::SymmetricRandom() * posFactor;
				node->setPosition(nodePos);

				cloneEnt = mKnotEnt->clone(cloneName);

				// Attach to new node
				node->attachObject(cloneEnt);
				rootNode->addChild(node);
			}
			mCurrNumClones += mNumClone;
		}
	}
	void removeStaticEntities()
	{
		if (mCurrNumClones > 0)
		{
			// detach the entity to the root of the scene
			SceneNode* rootNode = mCamera->getSceneManager()->getRootSceneNode();
			// Remove a whole bunch of extra entities
			mCurrNumClones -= mNumClone;
			for (unsigned int n = 0; n < mNumClone; ++n)
			{
				// get Clone knot name
				char cloneName[255];
				// StaticOccluder in the name is the way for the Scene manager to know how to handle it.
				sprintf(cloneName, "StaticOccluder_Knot%d", n + mCurrNumClones);

				rootNode->removeAndDestroyChild(cloneName);
				mCamera->getSceneManager()->destroyEntity(cloneName);
			}
		}
	}
	void switchToShaders()
	{
		static bool isShaders = false;

		isShaders = !isShaders;
		for (unsigned int n = 0; n < mCurrNumClones; ++n)
		{
			// get Clone knot name
			char cloneName[255];
			// StaticOccluder in the name is the way for the Scene manager to know how to handle it.
			sprintf(cloneName, "StaticOccluder_Knot%d", n);
			if (isShaders)
				mCamera->getSceneManager()->getEntity(cloneName)->setMaterialName("Examples/OffsetMapping/Specular");
			else
				mCamera->getSceneManager()->getEntity(cloneName)->setMaterialName("Examples/TextureEffect2");
		}
	}
};



class CHCDemoApplication : public ExampleApplication
{
public:

	CHCDemoApplication() :
		ExampleApplication(),
		mResourceGrpName ("CHC")
	{

	}

    ~CHCDemoApplication()
    {
	}

protected:

	const String mResourceGrpName; 

	// These internal methods package up the stages in the startup process
	/** Sets up the application - returns false if the user chooses to abandon configuration. */
	virtual bool setup(void)
	{
		mRoot = new Root("plugins.cfg", "CHC_display.cfg", "CHC.log");
#ifdef _DEBUG
		mRoot->loadPlugin("Plugin_OctreeCHCSceneManager_d");
#else
		mRoot->loadPlugin("Plugin_OctreeCHCSceneManager");
#endif
		setupResources();

		bool carryOn = configure();
		if (!carryOn) return false;

		chooseSceneManager();
		createCamera();
		createViewports();

		// Set default mipmap level (NB some APIs ignore this)
		TextureManager::getSingleton().setDefaultNumMipmaps(20);

		// Create any resource listeners (for loading screens)
		createResourceListener();
		// Load resources
		loadResources();

		// Create the scene
		createScene();

		createFrameListener();

		return true;

	}

	void setupResources()
	{
		ExampleApplication::setupResources();

		ResourceGroupManager *rsm = ResourceGroupManager::getSingletonPtr();
		StringVector groups = rsm->getResourceGroups();        
		if (std::find(groups.begin(), groups.end(), mResourceGrpName) == groups.end())
		{
			FileInfoListPtr finfo =  ResourceGroupManager::getSingleton().findResourceFileInfo (
				"Bootstrap", "axes.mesh");
			const bool isSDK =  (!finfo->empty()) && 
				StringUtil::startsWith (finfo->begin()->archive->getName(), "../../media/packs/ogrecore.zip", true);

			rsm->createResourceGroup(mResourceGrpName);

			if (isSDK)
			{
				rsm->addResourceLocation ("../../../CHC/Media","FileSystem", mResourceGrpName);
				rsm->addResourceLocation ("../../../CHC/Media/gui","FileSystem", mResourceGrpName);
				rsm->addResourceLocation ("../../../CHC/Media/materials","FileSystem", mResourceGrpName);
				rsm->addResourceLocation ("../../../CHC/Media/materials/scripts","FileSystem", mResourceGrpName);
			}
			else
			{
				rsm->addResourceLocation ("../../../../../ogreaddons/CHC/Media", "FileSystem", mResourceGrpName);
				rsm->addResourceLocation ("../../../../../ogreaddons/CHC/Media/gui", "FileSystem", mResourceGrpName);
				rsm->addResourceLocation ("../../../../../ogreaddons/CHC/Media/materials","FileSystem", mResourceGrpName);
				rsm->addResourceLocation ("../../../../../ogreaddons/CHC/Media/materials/scripts","FileSystem", mResourceGrpName);
			}
		}
		rsm->initialiseResourceGroup(mResourceGrpName);
	}


	//-----------------------------------------------------------------------
	void chooseSceneManager(void)
	{
#ifndef _STANDARD
		const String scnMgrName ("OctreeCHCSceneManager");
#else
		const String scnMgrName ("OctreeSceneManager");
#endif

		bool notFound = true;
		// Get the SceneManager, in this case the Paging LandScape specialization
		SceneManagerEnumerator::MetaDataIterator it = mRoot->getSceneManagerMetaDataIterator();
		while (it.hasMoreElements ())
		{
			const SceneManagerMetaData* metaData = it.getNext ();

			/// A mask describing which sorts of scenes this manager can handle
			if (metaData->sceneTypeMask == 0xFFFF  && 
				metaData->worldGeometrySupported == false &&
				metaData->typeName == scnMgrName

				)
			{
				notFound = false;
				break;
			}
		}
		if (notFound)
		{
			OGRE_EXCEPT(Exception::ERR_ITEM_NOT_FOUND, "Could not find " + scnMgrName + " plugin. Check if it's in plugins.cfg.",
				"chooseSceneManager");
		}

		mSceneMgr = mRoot->createSceneManager(scnMgrName, "CHCDemo" );
	}

	//-----------------------------------------------------------------------
    // Just override the mandatory create scene method
    void createScene(void)
    {

		ResourceGroupManager::getSingleton().loadResourceGroup(mResourceGrpName);

        // Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(0.2, 0.2, 0.2));

        // Skybox
        mSceneMgr->setSkyBox(true, "Examples/MorningSkyBox", 50000, false);


        // Create a light
        Light* l = mSceneMgr->createLight("MainLight");
        l->setType(Light::LT_DIRECTIONAL);
        Vector3 dir(0.5, -1, 0);
        dir.normalise();
        l->setDirection(dir);
        l->setDiffuseColour(1.0f, 1.0f, 0.8f);
        l->setSpecularColour(1.0f, 1.0f, 1.0f);


        mCamera->setPosition(-50, 100, 500);
        mCamera->lookAt(0,0,0);

    }

	//-----------------------------------------------------------------------
    void createFrameListener(void)
    {
        mFrameListener= new CHCDemoFrameListener(mWindow, mCamera);
        mRoot->addFrameListener(mFrameListener);

	}
	//-----------------------------------------------------------------------

};

#ifdef __cplusplus
extern "C" {
#endif
//-----------------------------------------------------------------------
int main(int argc, char ** argv)
{

    // Create application object
    CHCDemoApplication app;

    app.go();

    return 0;
}

#ifdef __cplusplus
}
#endif
