/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2006 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/
/***************************************************************************
OctreeCHC.h  -  description
-------------------
begin                : Mon Sep 30 2002
copyright            : (C) 2002 by Jon Anderson
email                : janders@users.sf.net

Enhancements 2003 - 2004 (C) The OGRE Team
***************************************************************************/

#ifndef __OctreeCHC_H
#define __OctreeCHC_H

#include "OctreeCHCPrerequisites.h"

#include "OgreAxisAlignedBox.h"

#include "OgreSpatialTree.h"
#include "OctreeCHCOcclusionElement.h"

#include <list>

namespace OgreOctreeCHC
{

	class OctreeCHCNode;

	typedef std::list < OctreeCHCNode* > OctreeCHCNodeList;


	/** OctreeCHC data structure for managing scene nodes.
	@remarks
		This is a loose OctreeCHC implementation, meaning that each
		octant child of the OctreeCHC actually overlaps it's siblings by a factor
		of .5.  This guarantees that any thing that is half the size of the parent will
		fit completely into a child, with no splitting necessary.
	*/

    class OctreeCHC : public SpatialTree
	{

	    public:
            OctreeCHC(SpatialTreeManager *treemanager);
			~OctreeCHC(void){};

		    
        	/** Determines if this OctreeCHC is twice as big as the given box.
		    @remarks
			    This method is used by the OctreeCHCSceneManager to determine if the given
			    box will fit into a child of this OctreeCHC.
		    */
		    bool _isTwiceSize(const Ogre::AxisAlignedBox& box) const;

            /** Determines if this OctreeCHC is twice as big as the given box.
		    @remarks
			    This method is used by the OctreeCHCSceneManager to determine if the given
			    box will fit into a child of this OctreeCHC.
		    */
		    bool _isTwiceCullSize(const Ogre::AxisAlignedBox& box) const;

            bool _isNotCrossingAxes(const Ogre::AxisAlignedBox& box) const;
			/**  Returns the appropriate indexes for the child of this OctreeCHC into which the box will fit.
		    @remarks
			    This is used by the OctreeCHCSceneManager to determine which child to traverse next when
			    finding the appropriate OctreeCHC to insert the box.  Since it is a loose OctreeCHC, only the
		    center of the box is checked to determine the octant.
		    */
		    OctreeCHC* _getChildWhereBoxFits(const Ogre::AxisAlignedBox& bx);
            /**  Returns the appropriate indexes for the child of this OctreeCHC into which the box will fit.
		    @remarks
			    This is used by the OctreeCHCSceneManager to determine which child to traverse next when
			    finding the appropriate OctreeCHC to insert the box.  Since it is a loose OctreeCHC, only the
		    center of the box is checked to determine the octant.
		    */
            OctreeCHC *_getCullChildWhereBoxFits(const Ogre::AxisAlignedBox& bx);
		 
            void setBoundingBox(const Ogre::Vector3 &min,  const Ogre::Vector3 &max);

			
            void _insertNode(SpatialTreeNode* node);
            void _insertMovableNode(OctreeCHCNode * const n, const int depth);
			void _insertStaticNode(OctreeCHCNode * const n, const int depth);

            inline const Ogre::Vector3        &getHalfSize()        const {return mHalfSize;};
            inline const Ogre::Vector3        &getCullHalfSize()    const {return mCullHalfSize;};

	    protected:
		    /** Vector containing the dimensions of this OctreeCHC 
		    */
		    Ogre::Vector3 mHalfSize;
		    /** Vector containing the dimensions of this OctreeCHC / 2
		    */
			Ogre::Vector3 mCullHalfSize;
			/** Vector containing the dimensions of this OctreeCHC 
			*/
			Ogre::AxisAlignedBox mDoubleBoundingBox;
	};

}

#endif //__OctreeCHC_H
