/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2006 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#ifndef _OgreOctreeCHCDebugRectangle2D_H__
#define _OgreOctreeCHCDebugRectangle2D_H__

#include "OctreeCHCPrerequisites.h"

#ifdef _VISIBILITYDEBUG

#include "OgreSimpleRenderable.h"

namespace OgreOctreeCHC 
{

    /** Allows the rendering of a simple 2D rectangle
    This class renders a simple 2D rectangle; this rectangle has no depth and
    therefore is best used with specific render queue and depth settings,
    like RENDER_QUEUE_BACKGROUND and 'depth_write off' for backdrops, and 
    RENDER_QUEUE_OVERLAY and 'depth_check off' for fullscreen quads.
    */
    class DebugRectangle2D : public Ogre::SimpleRenderable
    {
        protected:
            /** Override this method to prevent parent transforms (rotation,translation,scale)
            */
            void getWorldTransforms(Ogre::Matrix4* xform) const;
            /** @copydoc Renderable::getWorldOrientation */
            const Ogre::Quaternion& getWorldOrientation(void) const;
            /** @copydoc Renderable::getWorldPosition */
            const Ogre::Vector3& getWorldPosition(void) const;

        public:

            DebugRectangle2D();
            ~DebugRectangle2D();

            /** Sets the corners of the rectangle, in relative coordinates.
            @param
            left Left position in screen relative coordinates, -1 = left edge, 1.0 = right edge
            top Top position in screen relative coordinates, 1 = top edge, -1 = bottom edge
            right Right position in screen relative coordinates
            bottom Bottom position in screen relative coordinates
            */
            void setCorners(Ogre::Real left, Ogre::Real top, Ogre::Real right, Ogre::Real bottom);

            Ogre::Real getSquaredViewDepth(const Ogre::Camera* cam) const { return 0; }

            Ogre::Real getBoundingRadius(void) const { return 0; }
            /// Identity view and projection
            bool getUseIdentityProjection(void) const { return mUseIdentityProjection; }
            /// Identity view and projection
            bool getUseIdentityView(void) const { return mUseIdentityView; }

    };

}// namespace

#endif //_VISIBILITYDEBUG

#endif // _OgreOctreeCHCDebugRectangle2D_H__

