/***************************************************************************
OctreeCHCManager.h  -  description
-------------------
begin                : Fri Sep 27 2002
copyright            : (C) 2002 by Jon Anderson
email                : janders@users.sf.net
***************************************************************************/

/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2006 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#ifndef __OctreeCHCManager_H
#define __OctreeCHCManager_H

#include "OgreSceneManager.h"
#include "OgreRenderOperation.h"
#include "OgreSphere.h"

#include <list>
#include <algorithm>

#include "OctreeCHCPrerequisites.h"

#include "OctreeCHC.h"
#include "OctreeCHCOcclusion.h"
#include "OgreRenderable.h"

#include "OctreeCHCPoolSet.h"

namespace OgreOctreeCHC
{
    /** A collection of pre-allocated octrees
    */
    class OctreeCHCSet : public PoolSet<OctreeCHC> 
    {
	public:
		OctreeCHCSet() : PoolSet<OctreeCHC> () {}
		
		void setTreeManager(SpatialTreeManager *treeManager){mTreeManager = treeManager;};
    protected:
        inline OctreeCHC* allocate ()
        {
            return new OctreeCHC(mTreeManager);
        }
        inline void deallocate (OctreeCHC *p)
        {
            delete p;
        }
	private:
		SpatialTreeManager *mTreeManager;
    };

    /** Specialized TreeManager that divides the geometry into an OctreeCHC in order to facilitate spatial queries.
   */
    class OctreeCHCManager : public SpatialTreeManager
    {
    public:
        /** Standard Constructor */
		OctreeCHCManager(OctreeCHCSceneManager *scnMgr, 
			const Ogre::AxisAlignedBox &box, 
			int max_depth);
        /** Standard destructor */
        virtual ~OctreeCHCManager(void);

       inline SpatialTree * getNewTree()
        {
            return mOctreeSet.getPoolable();
        }
        inline void deleteTree(SpatialTree * p)
        {
            p->reset ();
            mOctreeSet.removePoolable (static_cast<OctreeCHC*> (p));
		}
		/** Creates a specialized SceneNode */
		void deleteTreeNode(SpatialTreeNode * p);  

        /** Creates a specialized OctreeCHCNode */
		SpatialTreeNode* getNewTreeNode(void);
		virtual SpatialTreeNode *getNewTreeNode(const Ogre::String &name);

        /** Checks the given SceneNode, and determines if it needs to be moved
        * to a different octant.
        */
        void _updateNode(SpatialTreeNode* node);         

      
    protected:

        OctreeCHCSet mOctreeSet;

    };


}

#endif //__OctreeCHCManager_H

