/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2006 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/
/***************************************************************************
OctreeCHCnode.h  -  description
-------------------
begin                : Fri Sep 27 2002
copyright            : (C) 2002 by Jon Anderson
email                : janders@users.sf.net

Enhancements 2003 - 2004 (C) The OGRE Team
***************************************************************************/
#ifndef __OctreeCHCNODE_H
#define __OctreeCHCNODE_H

#include "OctreeCHCPrerequisites.h"

#include "OgreSpatialTreeNode.h"

#include "OctreeCHCSceneManager.h"
#include "OgreSceneNode.h"
#include "OctreeCHCOcclusionElement.h"

namespace OgreOctreeCHC
{

    /** Specialized SceneNode that is customized for working within an OctreeCHC. Each node
    * maintains it's own bounding box, rather than merging it with all the children.
    *
    */
	class OctreeCHCNode : public SpatialTreeNode
    {
        public:
            /** Standard constructor */
            OctreeCHCNode(Ogre::SceneManager* creator);
            /** Standard constructor */
            OctreeCHCNode(Ogre::SceneManager* creator, const Ogre::String& name);
            /** Standard destructor */
            ~OctreeCHCNode(void);

            /** Determines if the center of this node is within the given box
            */
            bool _isIn(const Ogre::AxisAlignedBox& box) const;
           
            inline const Ogre::AxisAlignedBox &getCullBoundingBox() const ;

			inline const Ogre::Vector3 &getHalfSize()  const ;
            


        protected:
            /** Internal method for updating the bounds for this OctreeCHCNode.
            @remarks
				This method determines the bounds solely from the attached objects, not
				any children. If the node has changed it's bounds, it is removed from its
				current OctreeCHC, and reinserted into the tree.
            */
            void _updateBounds(void);

            Ogre::Vector3                 mHalfSize;
            // if a pre-notified object is visible.
            MovableObjectList       mNotifiedVisibles;
            
    };
}

#include "OctreeCHCSceneManager.h"
#include "OctreeCHC.h"

namespace OgreOctreeCHC
{
	//-----------------------------------------------------------------------
	//-----------------------------------------------------------------------
	//-----------------------------------------------------------------------
	inline const Ogre::AxisAlignedBox &OctreeCHCNode::getCullBoundingBox() const 
	{
		return _getWorldAABB();
	};
	//-----------------------------------------------------------------------
	inline  const Ogre::Vector3 &OctreeCHCNode::getHalfSize()  const 
	{
		return mHalfSize;
	};
}


#endif //__OctreeCHCNODE_H
