#ifndef __OctreeCHCOcclusionViewFrustumCullingDepthTraversal_H
#define __OctreeCHCOcclusionViewFrustumCullingDepthTraversal_H

#include "OctreeCHCPrerequisites.h"

#include "OgreRenderable.h"
#include "OgreWireBoundingBox.h"

#include "OctreeCHCCamera.h"

#include "OctreeCHCOcclusionTraversal.h"

namespace OgreOctreeCHC
{
    //-----------------------------------------------------------------------
    // Basic View Frustum Culling using hierarchy info 
    // as if an octree is fully in frustum, all children are for sure visible
	class ViewFrustumCullingDepthTraversal : public DepthTraversal 
    {
	    public:

			ViewFrustumCullingDepthTraversal (OcclusionCulling& o, Ogre::VisibleObjectsBoundsInfo * const visibleBounds):
				  mOcclusion(o),
				  mVisibleBounds(visibleBounds),
				  mIsVisible(OctreeCHCCamera::NONE)
			  {

			  };

            void onDepthTree(SpatialTree&);
            void onDepthLeaf (SpatialTreeNode&);
	    private:

		    OcclusionCulling& mOcclusion;
			OctreeCHCCamera::Visibility mIsVisible;
			Ogre::VisibleObjectsBoundsInfo * const mVisibleBounds;

	};
	//-----------------------------------------------------------------------
	// Basic View Frustum Culling using hierarchy info 
	// as if an octree is fully in frustum, all children are for sure visible
	class ViewFrustumCullingDepthTraversalPlaneMask : public DepthTraversal 
	{
	public:

		ViewFrustumCullingDepthTraversalPlaneMask (OcclusionCulling& o, Ogre::VisibleObjectsBoundsInfo * const visibleBounds):
			  mOcclusion(o),
			  mVisibleBounds(visibleBounds),
			  mIsVisible(OctreeCHCCamera::NONE),
			  mMask(65535)
		  {
			  // Skip far plane if infinite view frustum
			  if (mOcclusion.mCurrentCam->getFarClipDistance() == 0)
				mMask &= ~(Ogre::FRUSTUM_PLANE_FAR << 1);
		  };

		  void onDepthTree(SpatialTree&);
		  void onDepthLeaf (SpatialTreeNode&);
	private:

		OcclusionCulling& mOcclusion;
		OctreeCHCCamera::Visibility mIsVisible;
		Ogre::VisibleObjectsBoundsInfo * const mVisibleBounds;
		size_t mMask;

	};
	//-----------------------------------------------------------------------
	// Basic View Frustum Culling using hierarchy info 
	// as if an octree is fully in frustum, all children are for sure visible
	class ViewFrustumCullingDepthTraversalPlaneMaskTemporal : public DepthTraversal 
	{
	public:

		ViewFrustumCullingDepthTraversalPlaneMaskTemporal (OcclusionCulling& o, Ogre::VisibleObjectsBoundsInfo * const visibleBounds):
		  mOcclusion(o),
			  mVisibleBounds(visibleBounds),
			  mIsVisible(OctreeCHCCamera::NONE),
			  mMask(65535)
		  {
			  // Skip far plane if infinite view frustum
			  if (mOcclusion.mCurrentCam->getFarClipDistance() == 0)
				  mMask &= ~(Ogre::FRUSTUM_PLANE_FAR << 1);
		  };

		  void onDepthTree(SpatialTree&);
		  void onDepthLeaf (SpatialTreeNode&);
	private:

		OcclusionCulling& mOcclusion;
		OctreeCHCCamera::Visibility mIsVisible;
		Ogre::VisibleObjectsBoundsInfo * const mVisibleBounds;
		size_t mMask;

	};
	//-----------------------------------------------------------------------
	// Complex View Frustum Culling using hierarchy info and more camera information
	// as if an octree is fully in frustum, all children are for sure visible
	class ViewFrustumCullingDepthTraversalComplex : public DepthTraversal 
	{
	public:

		ViewFrustumCullingDepthTraversalComplex (OcclusionCulling& o, Ogre::VisibleObjectsBoundsInfo * const visibleBounds):
			  mOcclusion(o),
			  mVisibleBounds(visibleBounds),
			  mIsVisible(OctreeCHCCamera::NONE)
		  {

		  };

		  void onDepthTree(SpatialTree&);
		  void onDepthLeaf (SpatialTreeNode&);
	private:

		OcclusionCulling& mOcclusion;
		OctreeCHCCamera::Visibility mIsVisible;
		Ogre::VisibleObjectsBoundsInfo * const mVisibleBounds;

	};
    //-----------------------------------------------------------------------
    // Basic View Frustum Culling
	class ViewFrustumCullingDepthTraversalDirect : public DepthTraversal 
    {

	    public:

			ViewFrustumCullingDepthTraversalDirect (OcclusionCulling& o, Ogre::VisibleObjectsBoundsInfo * const visibleBounds)
				:
				mOcclusion(o),
				mVisibleBounds(visibleBounds)
			{

			};

            void onDepthTree(SpatialTree&);
            void onDepthLeaf (SpatialTreeNode&);
	    private:

			OcclusionCulling& mOcclusion;
			Ogre::VisibleObjectsBoundsInfo * const mVisibleBounds;
	};

	//-----------------------------------------------------------------------
	// Radar View Frustum Culling
	class ViewFrustumCullingDepthTraversalRadar : public DepthTraversal 
	{

	public:

		ViewFrustumCullingDepthTraversalRadar (OcclusionCulling& o, Ogre::VisibleObjectsBoundsInfo * const visibleBounds)
			:
			mOcclusion(o),
			mVisibleBounds(visibleBounds)
		{

		};

		void onDepthTree(SpatialTree&);
		void onDepthLeaf (SpatialTreeNode&);
	private:

		OcclusionCulling& mOcclusion;
		Ogre::VisibleObjectsBoundsInfo * const mVisibleBounds;
	};


	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversal::onDepthTree(SpatialTree& node)
	{
		mOcclusion.mTraversedNodesCount++;   

		OctreeCHCCamera::Visibility newVis = mIsVisible;
		if (newVis != OctreeCHCCamera::FULL)      
		{
			newVis = mOcclusion.mCurrentCam->getVisibility(node._getCullBounds());
			if (newVis == OctreeCHCCamera::NONE)
			{
				// Culled Octree as outside frustum                
				node.viewFrustumVisible = false;
				mOcclusion.mFrustumCulledNodesCount++;  
				return;
			}
		}         

		node.viewFrustumVisible = true;     

		if (mOcclusion.mShowBoxes)
		{
			mOcclusion.mCurrentRenderQueue->addRenderable(node.getWireBoundingBox());
		}

		// if we get there, then it's at least partially visible. 
		// Store parent Vis 
		const OctreeCHCCamera::Visibility oldVis = mIsVisible;
		mIsVisible = newVis;
		depthTraverseChildren (node);
		// Restore Parent Vis so that bothers can be pre-culled.
		mIsVisible = oldVis; 
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversal::onDepthLeaf (SpatialTreeNode &node)
	{
		mOcclusion.mTraversedNodesCount++;   

		// if this OctreeCHC is partially visible, manually cull all
		// scene nodes attached directly to this level.
		if (mIsVisible != OctreeCHCCamera::FULL)      
		{
			if (mOcclusion.mCurrentCam->getVisibility(node._getWorldAABB()) == OctreeCHCCamera::NONE)
			{
				// Culled Scene Node as outside frustum                
				node.viewFrustumVisible = false;
				mOcclusion.mFrustumCulledNodesCount++;  
				return;
			}
		}         
		node.viewFrustumVisible = true;
		mOcclusion.mObjectCount++;


		// if we get there, then it's at least partially visible.  
		node._findVisibleObjects (mOcclusion.mCurrentCam, 
			mOcclusion.mCurrentRenderQueue,
			mVisibleBounds,
			false, 
			mOcclusion.mDisplayNode,
			mOcclusion.mOnlyShadowCaster);       
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalPlaneMask::onDepthTree(SpatialTree& node)
	{
		mOcclusion.mTraversedNodesCount++;   

		OctreeCHCCamera::Visibility newVis = mIsVisible;
		size_t newMask = mMask;
		if (newVis != OctreeCHCCamera::FULL)      
		{
			newVis = mOcclusion.mCurrentCam->getHierarchicalVisibility(node, mMask, &newMask);
			if (newVis == OctreeCHCCamera::NONE)
			{
				// Culled Octree as outside frustum                
				node.viewFrustumVisible = false;
				mOcclusion.mFrustumCulledNodesCount++;  
				return;
			}
		}         

		node.viewFrustumVisible = true;     

		if (mOcclusion.mShowBoxes)
		{
			mOcclusion.mCurrentRenderQueue->addRenderable(node.getWireBoundingBox());
		}

		// if we get there, then it's at least partially visible. 
		// Store parent Vis 
		const OctreeCHCCamera::Visibility oldVis = mIsVisible;
		const size_t oldMask = mMask;
		mMask = newMask;
		depthTraverseChildren (node);
		// Restore Parent Vis so that bothers can be pre-culled.
		mMask = oldMask; 
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalPlaneMask::onDepthLeaf (SpatialTreeNode &node)
	{
		mOcclusion.mTraversedNodesCount++;   

		// if this OctreeCHC is partially visible, manually cull all
		// scene nodes attached directly to this level.
		if (mIsVisible != OctreeCHCCamera::FULL)      
		{
			size_t newMask;
			if (OctreeCHCCamera::NONE == mOcclusion.mCurrentCam->getHierarchicalVisibility(node, mMask, &newMask))
			{
				// Culled Scene Node as outside frustum                
				node.viewFrustumVisible = false;
				mOcclusion.mFrustumCulledNodesCount++;  
				return;
			}
		}         
		node.viewFrustumVisible = true;
		mOcclusion.mObjectCount++;


		// if we get there, then it's at least partially visible.  
		node._findVisibleObjects (mOcclusion.mCurrentCam, 
			mOcclusion.mCurrentRenderQueue,
			mVisibleBounds,
			false, 
			mOcclusion.mDisplayNode,
			mOcclusion.mOnlyShadowCaster);       
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalPlaneMaskTemporal::onDepthTree(SpatialTree& node)
	{
		mOcclusion.mTraversedNodesCount++;   

		OctreeCHCCamera::Visibility newVis = mIsVisible;
		size_t newMask = mMask;
		if (newVis != OctreeCHCCamera::FULL)      
		{
			node.currVis = node.getNodeData(mOcclusion.mCurrentCam);
			newVis = mOcclusion.mCurrentCam->getHierarchicalAndTemporalVisibility(node, mMask, &newMask);
			if (newVis == OctreeCHCCamera::NONE)
			{
				// Culled Octree as outside frustum                
				node.viewFrustumVisible = false;
				mOcclusion.mFrustumCulledNodesCount++;  
				return;
			}
		}         

		node.viewFrustumVisible = true;     

		if (mOcclusion.mShowBoxes)
		{
			mOcclusion.mCurrentRenderQueue->addRenderable(node.getWireBoundingBox());
		}

		// if we get there, then it's at least partially visible. 
		// Store parent Vis 
		const OctreeCHCCamera::Visibility oldVis = mIsVisible;
		const size_t oldMask = mMask;
		mMask = newMask;
		depthTraverseChildren (node);
		// Restore Parent Vis so that bothers can be pre-culled.
		mMask = oldMask; 
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalPlaneMaskTemporal::onDepthLeaf (SpatialTreeNode &node)
	{
		mOcclusion.mTraversedNodesCount++;   

		// if this OctreeCHC is partially visible, manually cull all
		// scene nodes attached directly to this level.
		if (mIsVisible != OctreeCHCCamera::FULL)      
		{
			size_t newMask;
			node.currVis = node.getNodeData(mOcclusion.mCurrentCam);
			if (OctreeCHCCamera::NONE == mOcclusion.mCurrentCam->getHierarchicalAndTemporalVisibility(node, mMask, &newMask))
			{
				// Culled Scene Node as outside frustum                
				node.viewFrustumVisible = false;
				mOcclusion.mFrustumCulledNodesCount++;  
				return;
			}
		}         
		node.viewFrustumVisible = true;
		mOcclusion.mObjectCount++;


		// if we get there, then it's at least partially visible.  
		node._findVisibleObjects (mOcclusion.mCurrentCam, 
			mOcclusion.mCurrentRenderQueue,
			mVisibleBounds,
			false, 
			mOcclusion.mDisplayNode,
			mOcclusion.mOnlyShadowCaster);       
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalComplex::onDepthTree(SpatialTree& node)
	{
		mOcclusion.mTraversedNodesCount++;   

		OctreeCHCCamera::Visibility newVis = mIsVisible;
		if (newVis != OctreeCHCCamera::FULL)      
		{
			newVis = mOcclusion.mCurrentCam->getVisibility(node);
			if (newVis == OctreeCHCCamera::NONE)
			{
				// Culled Octree as outside frustum                
				node.viewFrustumVisible = false;
				mOcclusion.mFrustumCulledNodesCount++;  
				return;
			}
		}         

		node.viewFrustumVisible = true;     

		if (mOcclusion.mShowBoxes)
		{
			mOcclusion.mCurrentRenderQueue->addRenderable(node.getWireBoundingBox());
		}

		// if we get there, then it's at least partially visible. 
		// Store parent Vis 
		const OctreeCHCCamera::Visibility oldVis = mIsVisible;
		mIsVisible = newVis;
		depthTraverseChildren (node);
		// Restore Parent Vis so that bothers can be pre-culled.
		mIsVisible = oldVis; 
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalComplex::onDepthLeaf (SpatialTreeNode &node)
	{
		mOcclusion.mTraversedNodesCount++;   

		// if this OctreeCHC is partially visible, manually cull all
		// scene nodes attached directly to this level.
		if (mIsVisible != OctreeCHCCamera::FULL)      
		{
			if (mOcclusion.mCurrentCam->getVisibility(node) == OctreeCHCCamera::NONE)
			{
				// Culled Scene Node as outside frustum                
				node.viewFrustumVisible = false;
				mOcclusion.mFrustumCulledNodesCount++;  
				return;
			}
		}         
		node.viewFrustumVisible = true;
		mOcclusion.mObjectCount++;


		// if we get there, then it's at least partially visible.  
		node._findVisibleObjects (mOcclusion.mCurrentCam, 
			mOcclusion.mCurrentRenderQueue,
			mVisibleBounds,
			false, 
			mOcclusion.mDisplayNode,
			mOcclusion.mOnlyShadowCaster);       
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalDirect::onDepthTree(SpatialTree& node)
	{
		mOcclusion.mTraversedNodesCount++;

		if (mOcclusion.mCurrentCam->isVisible(node._getCullBounds ()))      
		{         
			node.viewFrustumVisible = true;
			if (mOcclusion.mShowBoxes)
			{
				mOcclusion.mCurrentRenderQueue->addRenderable(node.getWireBoundingBox());
			}
			// if we get there, then it's at least partially visible.   
			depthTraverseChildren (node);     
		} 
		else
		{
			// Culled Octree.
			// outside frustum
			mOcclusion.mFrustumCulledNodesCount++;
			node.viewFrustumVisible = false;
		}                
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalDirect::onDepthLeaf (SpatialTreeNode &node)
	{
		mOcclusion.mTraversedNodesCount++;

		// if this OctreeCHC is partially visible, manually cull all
		// scene nodes attached directly to this level.
		if (mOcclusion.mCurrentCam->isVisible(node._getWorldAABB ()))      
		{                      
			node.viewFrustumVisible = true;
			mOcclusion.mObjectCount++;

			// if we get there, then it's at least partially visible.  
			node._findVisibleObjects (mOcclusion.mCurrentCam, 
				mOcclusion.mCurrentRenderQueue,
				mVisibleBounds,
				false, 
				mOcclusion.mDisplayNode,
				mOcclusion.mOnlyShadowCaster);

		} 
		else
		{
			// Culled Octree.
			// outside frustum
			mOcclusion.mFrustumCulledNodesCount++;
			node.viewFrustumVisible = false;
		}            
	}

	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalRadar::onDepthTree(SpatialTree& node)
	{
		mOcclusion.mTraversedNodesCount++;

		if (mOcclusion.mCurrentCam->isVisibleOnRadar(node))      
		{         
			node.viewFrustumVisible = true;
			if (mOcclusion.mShowBoxes)
			{
				mOcclusion.mCurrentRenderQueue->addRenderable((Ogre::Renderable *)node.getWireBoundingBox());
			}
			// if we get there, then it's at least partially visible.   
			depthTraverseChildren (node);     
		} 
		else
		{
			// Culled Octree.
			// outside frustum
			mOcclusion.mFrustumCulledNodesCount++;
			node.viewFrustumVisible = false;
		}                
	}
	//-----------------------------------------------------------------------
	inline void ViewFrustumCullingDepthTraversalRadar::onDepthLeaf (SpatialTreeNode &node)
	{
		mOcclusion.mTraversedNodesCount++;

		// if this OctreeCHC is partially visible, manually cull all
		// scene nodes attached directly to this level.
		if (mOcclusion.mCurrentCam->isVisibleOnRadar(node))      
		{                      
			node.viewFrustumVisible = true;
			mOcclusion.mObjectCount++;

			// if we get there, then it's at least partially visible.  
			node._findVisibleObjects (mOcclusion.mCurrentCam, 
				mOcclusion.mCurrentRenderQueue,
				mVisibleBounds,
				false, 
				mOcclusion.mDisplayNode,
				mOcclusion.mOnlyShadowCaster);

		} 
		else
		{
			// Culled Octree.
			// outside frustum
			mOcclusion.mFrustumCulledNodesCount++;
			node.viewFrustumVisible = false;
		}            
	}

}
#endif //__OctreeCHCOcclusionViewFrustumCullingDepthTraversal_H
