/***************************************************************************
OctreeCHCManager.h  -  description
-------------------
begin                : Fri Sep 27 2002
copyright            : (C) 2002 by Jon Anderson
email                : janders@users.sf.net
***************************************************************************/

/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2006 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#ifndef __OgreSpatialTreeManager_H
#define __OgreSpatialTreeManager_H

#include "OctreeCHCPrerequisites.h"

#include "OgreAxisAlignedBox.h"

namespace OgreOctreeCHC
{
    /** Abstract Tree Manager 
    */
    class SpatialTreeManager
    {
    public:
        
        /** Standard Constructor */
		SpatialTreeManager(OctreeCHCSceneManager *scnMgr, 
					const Ogre::AxisAlignedBox& box, 
					const int max_depth,
					const size_t maxChildrenCount) ;
        /** Standard destructor */
		virtual ~SpatialTreeManager(void);

        virtual SpatialTree * getNewTree() = 0;
        virtual void deleteTree(SpatialTree * p) = 0;   

		/** Creates a specialized SceneNode */
        virtual SpatialTreeNode * getNewTreeNode() = 0;
		virtual SpatialTreeNode * getNewTreeNode(const Ogre::String &name) = 0;
		virtual void deleteTreeNode(SpatialTreeNode * p) = 0;  

       
        /** Checks the given SceneNode, and determines if it needs to be moved inside the tree
        */
		virtual void _updateNode(SpatialTreeNode* node) = 0;
	    /** Removes the given SceneNode node */
        virtual void _removeNode(SpatialTreeNode* node);        
	    /** Adds the Tree Node, starting at the given Tree, and traversing at max to the specified depth.
        */
        virtual void _insertNode(SpatialTreeNode* node);	

        /** Resizes the OctreeCHC to the given size */
        virtual void resize(const Ogre::AxisAlignedBox& box);
        virtual void resize(const Ogre::AxisAlignedBox &box, const int depth);

		const Ogre::AxisAlignedBox &getBoundingBox()const {return mBoundingBox;};

		virtual void traverse(ConstDepthTraversalConst &traverser);
		virtual void traverse(DepthTraversalConst &traverser);
		virtual void traverse(DepthTraversal &traverser);

		// mMaxDepth getter
		int getMaxDepth() const { return mMaxDepth; }
		// mMaxDepth setter
		void setMaxDepth(int val) { mMaxDepth = val; }
		size_t getNumNodes() const ;
		inline size_t getMaxChildrenCount() const {return mMaxChildrenCount;}

		SpatialTree* getRoot(){return mTree;};
		OctreeCHCSceneManager* getSceneManager(){return mScnMgr;};

    protected:
        /// The root Tree
		OctreeCHCSceneManager* mScnMgr;
        /// The root Tree
        SpatialTree* mTree;
		// Each Tree can have at max that Count of children
		// Quadtree: 4, Octree: 8
		const size_t mMaxChildrenCount;
        /// max depth for the tree.
        int mMaxDepth;
        /// Size of the Tree
        Ogre::AxisAlignedBox mBoundingBox;
    };

}

#endif //__OgreSpatialTreeManager_H

