/*
-----------------------------------------------------------------------------
This source file is part of OGRE
(Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2006 The OGRE Team
Also see acknowledgements in Readme.html

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/
/***************************************************************************
OctreeCHCManager.cpp  -  description
-------------------
begin                : Fri Sep 27 2002
copyright            : (C) 2002 by Jon Anderson
email                : janders@users.sf.net
 
Enhancements 2003 - 2004 (C) The OGRE Team
 
***************************************************************************/

#include "OctreeCHCPrecompiledHeaders.h"

#include "OctreeCHCManager.h"

#include "OctreeCHCAxisAlignedBoxSceneQuery.h"
#include "OctreeCHCIntersectionSceneQuery.h"
#include "OctreeCHCPlaneBoundedVolumeListSceneQuery.h"
#include "OctreeCHCRaySceneQuery.h"
#include "OctreeCHCSphereSceneQuery.h"
#include "OgreRenderSystem.h"
#include "OgreWireBoundingBox.h"
#include "OgreRenderOperation.h"
#include "OgreStringConverter.h"

#include "OgreOverlayManager.h"
#include "OgreOverlayElement.h"

#include "OctreeCHCCamera.h"
#include "OctreeCHC.h"
#include "OctreeCHCNode.h"

#include "OctreeCHCOcclusionBoundingBox.h"
#include "OctreeCHCOcclusionSorter.h"
#include "OctreeCHCOcclusionVisibilityData.h"
#include "OctreeCHCOcclusionTraversal.h"
#include "OctreeCHCOcclusionCameraTraversal.h"
#include "OctreeCHCOcclusionVFTraversal.h"
#include "OctreeCHCOcclusionSWTraversal.h"
#include "OctreeCHCOcclusionDebugTraversal.h"


using namespace Ogre;

namespace OgreOctreeCHC
{
    //---------------------------------------------------------------------
    OctreeCHCManager::OctreeCHCManager(OctreeCHCSceneManager *scnMgr, 
		const AxisAlignedBox &box, 
		int max_depth): 
        SpatialTreeManager(scnMgr, box, max_depth, 8)
    {
		mOctreeSet.setTreeManager(this);
        mOctreeSet.setPoolSize (16);
    }
    //---------------------------------------------------------------------
    OctreeCHCManager::~OctreeCHCManager()
    {
        mOctreeSet.deletePool ();
		mTree = 0;
	}
	//---------------------------------------------------------------------
	SpatialTreeNode * OctreeCHCManager::getNewTreeNode(const Ogre::String &name)
	{
		return new OctreeCHCNode(mScnMgr, name);
	}
	//---------------------------------------------------------------------
	SpatialTreeNode * OctreeCHCManager::getNewTreeNode()
	{
		return new OctreeCHCNode(mScnMgr);
	}
	//---------------------------------------------------------------------
	void OctreeCHCManager::deleteTreeNode(SpatialTreeNode * p)
	{
		delete p;
	}
    //---------------------------------------------------------------------
    void OctreeCHCManager::_updateNode(SpatialTreeNode * tnode)
    {
		OctreeCHCNode *onode = static_cast <OctreeCHCNode *> (tnode);
        const Ogre::AxisAlignedBox &box (onode->_getWorldAABB());


        if (onode->getTreeParent() == 0)
        {
            //if outside the OctreeCHC, force into the root node.
            if (! onode->_isIn(mTree->getBoundingBox()))
                mTree->_addNode(onode);
            else
                mTree->_insertNode(onode);
            return ;
        }

        if (! onode->_isIn(onode->getTreeParent()->getBoundingBox()))
        {
            _removeNode(onode);

            //if outside the OctreeCHC, force into the root node.
            if (! onode->_isIn(mTree->getBoundingBox()))
                mTree->_addNode(onode);
            else
                mTree->_insertNode(onode);
        }
    }
}





